/*
enum ByteCode {
    Report = 0x00,
    Collection = 0xA1,
    UsagePage = 0x05,
    Usage = 0x09,
    UsageMin = 0x19,
    UsageMax = 0x29,
    LogicalMin = 0x15,
    LogicalMax = 0x25,
    PhysicalMin = 0x100,
    PhysicalMax = 0x101,
    ReportSize = 0x75,
    ReportCount = 0x95,
}

const F_TYPE       :u16 = 0x03;
const TYPE_INPUT   :u16 = 0x01;
const TYPE_OUTPUT  :u16 = 0x02;
const TYPE_FEATURE :u16 = 0x03;
const F_DC         :u16 = 0x40; // Data / Const
const F_RA         :u16 = 0x80; // Relative / Absolute

struct _Usage {
    usage_page:u16,
    usage:u16,
    offset:u16,
    size:u16,
    fields:u16,
}
*/


// Main Items
const MN_INPUT            :u8 = 0b1000_00_00;
const MN_OUTPUT           :u8 = 0b1001_00_00;
const MN_FEATURE          :u8 = 0b1011_00_00;
const MN_COLLECTION_BEGIN :u8 = 0b1010_00_00;
const MN_COLLECTION_END   :u8 = 0b1100_00_00;
const MN_EXTENDED         :u8 = 0b1111_11_00;

// Global Items
const GB_USAGE_PAGE    :u8 = 0b0000_01_00;
const GB_LOGICAL_MIN   :u8 = 0b0001_01_00;
const GB_LOGICAL_MAX   :u8 = 0b0010_01_00;
const GB_PHYSICAL_MIN  :u8 = 0b0011_01_00;
const GB_PHYSICAL_MAX  :u8 = 0b0100_01_00;
const GB_UNIT_EXPONENT :u8 = 0b0101_01_00;
const GB_UNIT          :u8 = 0b0110_01_00;
const GB_REPORT_SIZE   :u8 = 0b0111_01_00;
const GB_REPORT_ID     :u8 = 0b1000_01_00;
const GB_REPORT_COUNT  :u8 = 0b1001_01_00;
const GB_PUSH          :u8 = 0b1010_01_00;
const GB_POP           :u8 = 0b1011_01_00;

// Local Items
const LC_USAGE            :u8 = 0b0000_10_00;
const LC_USAGE_MIN        :u8 = 0b0001_10_00;
const LC_USAGE_MAX        :u8 = 0b0010_10_00;
const LC_DESIGNATOR_INDEX :u8 = 0b0011_10_00;
const LC_DESIGNATOR_MIN   :u8 = 0b0100_10_00;
const LC_DESIGNATOR_MAX   :u8 = 0b0101_10_00;
const LC_STRING_INDEX     :u8 = 0b0111_10_00;
const LC_STRING_MIN       :u8 = 0b1000_10_00;
const LC_STRING_MAX       :u8 = 0b1001_10_00;
const LC_DELIMITER        :u8 = 0b1010_10_00;

struct Context {
    usage_page:u16,
    usages:Vec<u16>,
    logical_min:i32,
    logical_max:i32,
    physical_min:Option<i32>,
    physical_max:Option<i32>,
    report_count:u16,
    report_size:u16,
}

struct GlobalState {
    usage_page:u32,
    logical_min:i32,
    logical_max:i32,
    physical_min:i32,
    physical_max:i32,
    unit_exponent:i32,
    unit:u32,
    report_id:u32,
    report_size:u32,
    report_count:u32,
}
impl GlobalState {
    pub fn new() -> Self
    {
        Self {
            usage_page:0,
            logical_min:0,
            logical_max:0,
            physical_min:0,
            physical_max:0,
            unit_exponent:0,
            unit:0,
            report_id:0,
            report_size:0,
            report_count:0,
        }
    }
}


pub struct ReportDescriptor {
    
}
impl ReportDescriptor {
    pub fn parse(bytes:&[u8]) -> Result<Self,()>
    {
        // Output
        let mut descriptor = Self {

        };

        // Global State
        let mut global_state = Vec::<GlobalState>::new();
        global_state.push(GlobalState::new());

        // Local State
        let usages = Vec::<[u16;2]>::new();

        // Tracking
        let mut collection_index = 0;
        let mut offset = 0;
        let mut report_id = 0;

        let mut index  = 0;
        while index < bytes.len() {
            let mut sz = bytes[index] & 0x03;
            let mut tg = bytes[index] & 0xFC;

            // Get byte size from index
            sz = [ 0, 1, 2, 4 ][sz as usize];

            index += 1;

            if bytes.len() - index >= sz as usize {

                // Handle long tag
                if sz == 2 && tg == MN_EXTENDED {
                    sz = bytes[index + 1];
                    tg = bytes[index + 2];

                    if bytes.len() - index < sz as usize {
                        return Err(());
                    }
                }
                
                if sz <= 4 {
                    match tg {
                        MN_INPUT => {
                            /*
                            ** 0: Data(0), Constant(1)
                            ** 1: Array(0), Variable(1)
                            ** 2: Absolute(0), Relative(1)
                            ** 3: NoWrap(0), Wrap(1)
                            ** 4: Linear(0), NonLinear(1)
                            ** 5: PreferredState(0), NoPreferred(1)
                            ** 6: NoNull(0), NullState(1)
                            ** 7: reserved
                            ** 8: BitField(0), BufferedBytes(1)
                            ** _: reserved
                            */

                            let mut is_const = false;
                            let mut is_var = false;
                            let mut is_rel = false;
                            let mut is_wrap = false;
                            let mut is_linear = false;
                            let mut is_pref = false;
                            let mut has_null = false;
                            let mut is_volatile = false;
                            let mut is_buffered = false;

                            if sz >= 1 {
                                is_const = (bytes[index + 1] & 0x01) != 0;
                                is_var = (bytes[index + 1] & 0x02) != 0;
                                is_rel = (bytes[index + 1] & 0x04) != 0;
                            }

                            if sz >= 2 {

                            }
                        }

                        MN_OUTPUT => {
                            /*
                            ** 0: Data(0), Constant(1)
                            ** 1: Array(0), Variable(1)
                            ** 2: Absolute(0), Relative(1)
                            ** 3: NoWrap(0), Wrap(1)
                            ** 4: Linear(0), NonLinear(1)
                            ** 5: PreferredState(0), NoPreferred(1)
                            ** 6: NoNull(0), NullState(1)
                            ** 7: NonVolatile(0), Volatile(1)
                            ** 8: BitField(0), BufferedBytes(1)
                            ** _: reserved
                            */

                        }

                        MN_FEATURE => {
                            /*
                            ** 0: Data(0), Constant(1)
                            ** 1: Array(0), Variable(1)
                            ** 2: Absolute(0), Relative(1)
                            ** 3: NoWrap(0), Wrap(1)
                            ** 4: Linear(0), NonLinear(1)
                            ** 5: PreferredState(0), NoPreferred(1)
                            ** 6: NoNull(0), NullState(1)
                            ** 7: NonVolatile(0), Volatile(1)
                            ** 8: BitField(0), BufferedBytes(1)
                            ** _: reserved
                            */

                        }

                        MN_COLLECTION_BEGIN => {
                            /*
                            ** 00: Physical
                            ** 01: Application
                            ** 02: Logical
                            ** 03: Report
                            ** 04: NamedArray
                            ** 05: UsageSwitch
                            ** 06: UsageModifier
                            ** 07-7F: reserved
                            ** 80-FF: vendor-defined
                            */
                            
                        }

                        MN_COLLECTION_END => {
                            
                        }

                        GB_USAGE_PAGE => {

                        }

                        GB_LOGICAL_MIN => {

                        }

                        GB_LOGICAL_MAX => {

                        }

                        GB_PHYSICAL_MIN => {

                        }

                        GB_PHYSICAL_MAX => {

                        }

                        GB_UNIT_EXPONENT => {

                        }

                        GB_UNIT => {

                        }

                        GB_REPORT_SIZE => {

                        }

                        GB_REPORT_ID => {

                        }

                        GB_REPORT_COUNT => {

                        }

                        GB_PUSH => {

                        }

                        GB_POP => {

                        }

                        LC_USAGE => {

                        }

                        LC_USAGE_MIN => {

                        }

                        LC_USAGE_MAX => {

                        }

                        LC_DESIGNATOR_INDEX => {

                        }

                        LC_DESIGNATOR_MIN => {

                        }

                        LC_DESIGNATOR_MAX => {

                        }

                        LC_STRING_INDEX => {

                        }

                        LC_STRING_MIN => {

                        }

                        LC_STRING_MAX => {

                        }

                        LC_DELIMITER => {

                        }

                        _ => { return Err(()); }
                    }
                }

                index += sz as usize;
            }
        }

        Ok(descriptor)
    }
}
